@@ -1,4 +1,5 @@
 import logging
+from json.decoder import JSONDecodeError
 from typing import Union, List
 
 from django.conf import settings
@@ -11,6 +12,7 @@ from rest_framework.reverse import reverse_lazy
 from rest_framework.status import HTTP_500_INTERNAL_SERVER_ERROR
 
 from .decorators import request_shell
+from .exceptions import MicroserviceException
 
 logger = logging.getLogger(__name__)
 
@@ -118,7 +120,7 @@ class ConnectionService:
     @request_shell
     def put(self, data: dict, **kwargs):
         request_data = self._requested_data()
-        request_data.update(data=data)
+        request_data.update(json=data)
         return self.host.session.put(**request_data)
 
     @request_shell
@@ -134,12 +136,12 @@ class ConnectionService:
         return self.host.session.post(**request_data)
 
     def service_response(self, method: str = None, **kwargs):
-        response = self.request_to_service(method=method, **kwargs)
-        if not getattr(response, 'status_code', None):
-            logger.error(self.connection_refused_error(response))
-            self.error_handler(method, response)
+        self.response = self.request_to_service(method=method, **kwargs)
+        if not getattr(self.response, 'status_code', None):
+            logger.error(self.connection_refused_error(self.response))
+            self.error_handler(method, self.response)
             return Response({'detail': 'connection refused'}, status=self.error_status_code)
-        return Response(response.json(), status=response.status_code)
+        return Response(self._response(), status=self.response.status_code)
 
     def request_to_service(self, method: str, **kwargs) -> RequestResponse:
         if method.lower() in self.http_method_names or self.get_additional_method_names():
@@ -149,6 +151,15 @@ class ConnectionService:
         kwargs['method'] = method
         return handler(**kwargs)
 
+    def _response(self):
+        try:
+            return self.response.json()
+        except JSONDecodeError:
+            return self.error_process()
+
+    def error_process(self):
+        raise MicroserviceException(self.response.content)
+
 
 class MicroServiceConnect(ConnectionService):
     SEND_COOKIES: bool = getattr(settings, 'REQUEST_SEND_COOKIES', False)
@@ -162,7 +173,7 @@ class MicroServiceConnect(ConnectionService):
 
     @classmethod
     def microservice_response(cls, request, reverse_url: str, **kwargs):
-        url = cls.reverse_url(reverse_url)
+        url = cls.reverse_url(reverse_url, kwargs=kwargs.pop('url_kwargs', {}))
         service = cls(request, url, special_headers=kwargs.pop('special_headers', {}))
         method = kwargs.pop('method', None) or request.method
         return service.service_response(method=method, **kwargs)
@@ -180,7 +191,7 @@ class MicroServiceConnect(ConnectionService):
         return self.request.COOKIES
 
     def _requested_data(self) -> dict:
-        data = super(MicroServiceConnect, self)._requested_data()
+        data = super()._requested_data()
         data.update({'cookies': self.get_cookies() if self.SEND_COOKIES else None})
         return data
 
@@ -199,7 +210,7 @@ class MicroServiceConnect(ConnectionService):
     @request_shell
     def put(self, data: dict = None, **kwargs):
         request_data = self._requested_data()
-        request_data.update(data=data or self.request.data)
+        request_data.update(json=data or self.request.data)
         return self.host.session.put(**request_data)
 
     @request_shell
